@@ -1746,6 +1746,8 @@ a trapper module like
         $Log::Log4perl::caller_depth--;
     }
 
+    1;
+
 and a C<tie> command in the main program to tie STDERR to the trapper
 module along with regular Log::Log4perl initialization:
 
@@ -1761,7 +1763,7 @@ module along with regular Log::Log4perl initialization:
          layout => "%d %M: %m%n",
         });
 
-    tie *STDERR, Trapper;
+    tie *STDERR, "Trapper";
     
 Make sure not to use STDERR as Log::Log4perl's file appender
 here (which would be the default in C<:easy> mode), because it would 
@@ -2426,6 +2428,124 @@ handle this error well and spits out a slew of confusing warning
 messages. But now you know, just use the correct module name and
 you'll be fine.
 
+=head2 Log4perl complains that no initialization happened during shutdown!
+
+If you're using Log4perl log commands in DESTROY methods of your objects,
+you might see confusing messages like
+
+    Log4perl: Seems like no initialization happened. Forgot to call init()?
+    Use of uninitialized value in subroutine entry at
+    /home/y/lib/perl5/site_perl/5.6.1/Log/Log4perl.pm line 134 during global
+    destruction. (in cleanup) Undefined subroutine &main:: called at
+    /home/y/lib/perl5/site_perl/5.6.1/Log/Log4perl.pm line 134 during global
+    destruction.
+
+when the program shuts down. What's going on? 
+
+This phenomenon happens if you have circular references in your objects, 
+which perl can't clean up when an object goes out of scope but waits
+until global destruction instead. At this time, however, Log4perl has
+already shut down, so you can't use it anymore.
+
+For example, here's a simple class which uses a logger in its DESTROY
+method:
+
+    package A;
+    use Log::Log4perl qw(:easy);
+    sub new { bless {}, shift }
+    sub DESTROY { DEBUG "Waaah!"; }
+
+Now, if the main program creates a self-referencing object, like in
+
+    package main;
+    use Log::Log4perl qw(:easy);
+    Log::Log4perl->easy_init($DEBUG);
+
+    my $a = A->new();
+    $a->{selfref} = $a;
+
+then you'll see the error message shown above during global destruction.
+How to tackle this problem?
+
+First, you should clean up your circular references before global 
+destruction. They will not only cause objects to be destroyed in an order
+that's hard to predict, but also eat up memory until the program shuts
+down.
+
+So, the program above could easily be fixed by putting
+
+    $a->{selfref} = undef;
+
+at the end or in an END handler. If that's hard to do, use weak references:
+
+    package main;
+    use Scalar::Util qw(weaken);
+    use Log::Log4perl qw(:easy);
+    Log::Log4perl->easy_init($DEBUG);
+
+    my $a = A->new();
+    $a->{selfref} = weaken $a;
+
+This allows perl to clean up the circular reference when the object 
+goes out of scope, and doesn't wait until global destruction.
+
+=head2 How can I access POE heap values from Log4perl's layout?
+
+POE is a framework for creating multitasked applications running in a
+single process and a single thread. POE's threads equivalents are
+'sessions' and since they run quasi-simultaneously, you can't use
+Log4perl's global NDC/MDC to hold session-specific data.
+
+However, POE already maintains a data store for every session. It is called
+'heap' and is just a hash storing session-specific data in key-value pairs.
+To access this per-session heap data from a Log4perl layout, define a
+custom cspec and reference it with the newly defined pattern in the layout:
+
+    use strict;
+    use POE;
+    use Log::Log4perl qw(:easy);
+
+    Log::Log4perl->init( \ q{
+        log4perl.logger = DEBUG, Screen
+        log4perl.appender.Screen = Log::Log4perl::Appender::Screen
+        log4perl.appender.Screen.layout = PatternLayout
+        log4perl.appender.Screen.layout.ConversionPattern = %U %m%n
+        log4perl.PatternLayout.cspec.U = \
+            sub { POE::Kernel->get_active_session->get_heap()->{ user } }
+    } );
+
+    for (qw( Huey Lewey Dewey )) {
+        POE::Session->create(
+            inline_states => {
+                _start    => sub {
+                    $_[HEAP]->{user} = $_;
+                    POE::Kernel->yield('hello');
+                },
+                hello     => sub {
+                    DEBUG "I'm here now";
+                }
+            }
+        );
+    }
+
+    POE::Kernel->run();
+    exit;
+
+The code snippet above defines a new layout placeholder (called
+'cspec' in Log4perl) %U which calls a subroutine, retrieves the active
+session, gets its heap and looks up the entry specified ('user').
+
+Starting with Log::Log4perl 1.20, cspecs also support parameters in 
+curly braces, so you can say
+
+    log4perl.appender.Screen.layout.ConversionPattern = %U{user} %U{id} %m%n
+    log4perl.PatternLayout.cspec.U = \
+            sub { POE::Kernel->get_active_session-> \
+                  get_heap()->{ $_[0]->{curlies} } }
+
+and print the POE session heap entries 'user' and 'id' with every logged
+message. For more details on cpecs, read the PatternLayout manual.
+
 =cut
 
 =head1 SEE ALSO
